package de.boreeas.motd.main;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bukkit.entity.Player;
import com.nijiko.permissions.PermissionHandler;
import java.io.FileOutputStream;
import java.util.HashMap;
import org.bukkit.ChatColor;

/**
 * @author Malte
 */
class MOTDParser {
    
    public boolean valid = true;                                                //Has the file been found, is everything okay
    public static PermissionHandler permHandler;                                //Check for groups
    
    private MOTD plugin;                                                        //Instance
    
    private NewsFileReader newsreader;
    private BufferedReader reader;                                              //Readers for properties files
    private FileOutputStream out;
    private File loc, dir;                                                      //Files for properties
    private Properties propMOTD = new Properties();                             //Property files
    
    private Calendar c = Calendar.getInstance();                                //Calendar for date
    private SimpleDateFormat sdf = new SimpleDateFormat("hh:mm");
    
    private HashMap<Short, ChatColor> colortable = new HashMap<Short, ChatColor>();
    
    private String worldname;                                                   //The groups to find the right MOTD
    
    
    public MOTDParser(MOTD instance) {
        try {
            plugin = instance;
            dir = plugin.getDataFolder();
            loc = new File(dir.getPath() + "/motd.properties");
            reader = new BufferedReader(new InputStreamReader(new FileInputStream(loc)));
            out = new FileOutputStream(loc);
            propMOTD.load(reader);
            permHandler = plugin.getPermHandler();
            worldname = instance.getServer().getWorlds().get(0).getName();
            
            newsreader = new NewsFileReader(plugin);
            
            fillColorTable();
            
            if (permHandler == null) {
                
                valid = false;
            }
            
        } catch (FileNotFoundException nfe) {
            MOTD.logger.log(Level.WARNING, "Could not find MOTD message file, attempting to create empty file");
            dir.mkdir();
            try {
                loc.createNewFile();
                reader = new BufferedReader(new InputStreamReader(new FileInputStream(loc)));
                propMOTD.load(reader);
            } catch (IOException ex) {
                MOTD.logger.log(Level.SEVERE, "Failed to create directory, disabling MOTD...", ex);
                valid = false;
            }
        } catch (IOException ioe) {
            
            MOTD.logger.log(Level.SEVERE, "Unexpected IOError ocurred, please report this to the plugin creator", ioe);
            valid = false;
        } 
    }
    
    
    /**
     * Returns a parsed MOTD
     * 
     * @param player - The player to be greeted
     * @return The parsed MOTD, each array representing one line;
     */
    public String[] getMessage(Player player) {
        if (!valid) {
            
            return new String[0];
        }       
        String group = permHandler.getPrimaryGroup(worldname, player.getName());
        String unparsed = propMOTD.getProperty(group, "");
        if (unparsed.equalsIgnoreCase("")) {
            
            unparsed = propMOTD.getProperty("Default", "");
        }
        return parse(unparsed, player);
        
    }
    
    
    //Replaces the parse instructions with right strings
    private String[] parse(String unparsed, Player player) {
        
        if (unparsed.equalsIgnoreCase("")) {
            
            return new String[0];
        }
        if (!valid) {
            
            return new String[0];
        }
        
        Player[] onlinelist = player.getServer().getOnlinePlayers();
        String onlineRepl = "";
        String adminRepl = "";
        
        for (int i = 0; i < onlinelist.length; i++) {
            
            onlineRepl += (onlinelist[i].getName() + ", ");
            if (onlinelist[i].isOp()) {
                
                adminRepl += (onlinelist[i].getName() + ", ");
            }
        }
        
        String group = permHandler.getPrimaryGroup(worldname, player.getName());
        try {
            onlineRepl = onlineRepl.substring(0, onlineRepl.length() - 2);
        } catch (StringIndexOutOfBoundsException oobe) {
            onlineRepl = null;
        }
        try {
            adminRepl = adminRepl.substring(0, adminRepl.length() - 2);
        } catch (StringIndexOutOfBoundsException oobe) {
            adminRepl = null;
        }
        String playRepl = player.getName();
        String timeRepl = sdf.format(c.getTime());
        String rankRepl = group;
        String newsRepl = newsreader.readAll();
        
        if (newsRepl != null) {
            
            unparsed = unparsed.replaceAll("%news%", newsRepl);
        }
        else {
            
            unparsed = unparsed.replaceAll("%news", "");
        }
        
        unparsed = unparsed.replaceAll("%online%", onlineRepl).replaceAll("%admin%", adminRepl)
                .replaceAll("%name%", playRepl).replaceAll("%time%", timeRepl).replaceAll("%rank%", rankRepl);
        
        
        unparsed = unparsed.replaceAll("null", "");
        
        for (short i = 0; i < 16; i++) {
            
            String split = "%" + Short.toString(i) + "%";
            String[] temp = unparsed.split(split);
            unparsed = temp[0];
            for (int j = 1; j < temp.length; j++) {
                
                unparsed += (colortable.get(Short.valueOf(i)) + temp[j]);
            }
        }
        return unparsed.split("%n%");
    }
    
    public void setMOTD(String[] args) throws IOException, ArrayIndexOutOfBoundsException{

        String rank = args[1];
        String motd = args[2];
            
        for (int i = 3; i < args.length; i++) {
                
            motd += (" " + args[i]);
        }
        propMOTD.setProperty(rank, motd);
        propMOTD.store(out, "MOTD file");
    }

    private void fillColorTable() {
        colortable.put(Short.valueOf((short)0), ChatColor.BLACK);
        colortable.put(Short.valueOf((short)1), ChatColor.DARK_BLUE);
        colortable.put(Short.valueOf((short)2), ChatColor.DARK_GREEN);
        colortable.put(Short.valueOf((short)3), ChatColor.DARK_AQUA);
        colortable.put(Short.valueOf((short)4), ChatColor.DARK_RED);
        colortable.put(Short.valueOf((short)5), ChatColor.DARK_PURPLE);
        colortable.put(Short.valueOf((short)6), ChatColor.GOLD);
        colortable.put(Short.valueOf((short)7), ChatColor.GRAY);
        colortable.put(Short.valueOf((short)8), ChatColor.DARK_GRAY);
        colortable.put(Short.valueOf((short)9), ChatColor.BLUE);
        colortable.put(Short.valueOf((short)10), ChatColor.GREEN);
        colortable.put(Short.valueOf((short)11), ChatColor.AQUA);
        colortable.put(Short.valueOf((short)12), ChatColor.RED);
        colortable.put(Short.valueOf((short)13), ChatColor.LIGHT_PURPLE);
        colortable.put(Short.valueOf((short)14), ChatColor.YELLOW);
        colortable.put(Short.valueOf((short)15), ChatColor.WHITE);
    }
    
}
